<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;

use Teknisa\FIN\Modules\MovimentacaoBancaria\Factories\Bco54500ConciliacaoFactory;

class Bco54500ConciliacaoAutomaticaService {

    protected $newCode;
    protected $entityManager;
    protected $Bco54500Conciliacao;

    public function __construct(Bco54500ConciliacaoFactory $factory){
        $this->Bco54500Conciliacao = $factory;
    }

    /**
     * Essa função é responsável por pegar as movimentações que serão conciliadas. Caso não haja movimentações
     * é verificado se existem lançamentos para semrem conciliados, para informar ao usuário. 
     * @param  Array $row array associativo com os parametros vindos do frondent
     * @return Array      array associativo com os lançamentos que devem ser conciliados
     */
    public function getRegisForConcilia($row) {
        $movs = $this->Bco54500Conciliacao->getMovsForConcili($row['NRDIA'], $row['TDTCONCILIA'], $row['DTCONCILIA'], $row['CDREDUCONT']);
        if (empty($movs)) {
            $registros = $this->Bco54500Conciliacao->verifyRegisForConcili($row['NRDIA'], $row['TDTCONCILIA'], $row['DTCONCILIA'], $row['CDREDUCONT']);
            if (empty($registros)) {
                throw new \Exception("Não há dados para conciliação.", 1);
            } else {
                throw new \Exception("Não há dados para conciliação automática, porém ainda existem lançamentos a serem conciliados. Favor verificar na Conciliação Manual.", 1);
            }
        } else {
            $movreais = [];
            $movbancs = [];
            
            $movreais_remove = [];
            $movbancs_remove = [];
            
            $movs_insert = [];
            
            foreach($movs as $mov){
                $movreal = $mov['CDREDUCONT'].$mov['CDSEQUEXTR'].$mov['NRSEQMOVREAL'];
                if (!in_array($movreal, $movreais)) { 
                    array_push($movreais, $movreal);
                }else{
                    array_push($movreais_remove, $movreal);
                }
                
                $movbanc = $mov['NRMOVBANC'];
                if (!in_array($movbanc, $movbancs)) { 
                    array_push($movbancs, $movbanc);
                }else{
                    array_push($movbancs_remove, $movbanc);
                }
            }
            
            foreach($movs as $mov){
                $movreal = $mov['CDREDUCONT'].$mov['CDSEQUEXTR'].$mov['NRSEQMOVREAL'];
                $movbanc = $mov['NRMOVBANC'];
                if ((!in_array($movreal, $movreais_remove)) && (!in_array($movbanc, $movbancs_remove))) { 
                    array_push($movs_insert, $mov);
                }
            }
            
            if (empty($movs_insert)) {
                throw new \Exception("Não há dados para conciliação automática, porém ainda existem lançamentos a serem conciliados. Favor verificar na Conciliação Manual.", 1);
            }
            
            return $movs_insert;
        }
    }

    /**
     * Essa função tem como finalidade updatar os registro que devem ser conciliados. Essa função é a concialção, prorpiamente dita.
     * 
     * @param  [type] $registros [description]
     * @return [type]            [description]
     */
    public function updateTables($registros) {
        for ($i=0; $i < count($registros); $i++) { 
            $this->Bco54500Conciliacao->updateMovrealAuto($registros[$i]['NRMOVBANC'], $registros[$i]['CDREDUCONT'], $registros[$i]['NRSEQMOVREAL'], $registros[$i]['CDSEQUEXTR']);
            $this->Bco54500Conciliacao->updateMovBancaio($registros[$i]['CDSEQUEXTR'], $registros[$i]['NRSEQMOVREAL'], $registros[$i]['CDREDUCONT'], $registros[$i]['NRMOVBANC']);
        }
    }

    /**
     * Essa função tem como finalidade verificar se ainda existem lançamentos a serem cinciliados a´pos a conciliação automática. 
     * Ira retornar true se tiver lançamentos e false se não tiver lançamentos.
     * 
     * @param  [type] $row [description]
     * @return [type]      [description]
     */
    public function existsMoreLancs($row) {
        $registros = $this->Bco54500Conciliacao->getMoreLancs($row['NRDIA'], $row['TDTCONCILIA'], $row['DTCONCILIA'], $row['CDREDUCONT']);
        if(empty($registros)) {
            return false;
        } else {
            return true;
        }
    }
}
